package yrpg.bamp.basedata.webapi;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.openapi.api.result.ApiSaveItemData;
import kd.bos.openapi.api.result.ApiSaveResult;
import kd.bos.openapi.api.result.ApiSaveRowErrorData;
import kd.bos.openapi.api.result.BaseFilterResult;
import kd.bos.openapi.api.result.BaseFilterItemData;
import kd.bos.openapi.common.custom.annotation.ApiController;
import kd.bos.openapi.common.custom.annotation.ApiParam;
import kd.bos.openapi.common.custom.annotation.ApiPostMapping;
import kd.bos.openapi.common.custom.annotation.ApiResponseBody;
import kd.bos.openapi.common.result.CustomApiResult;
import kd.bos.openapi.common.result.OpenApiResult;
import kd.bos.openapi.common.util.OpenApiSdkUtil;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;


@ApiController(value="GoodsBill",desc="统一url入口")
public class GoodsController {
    @ApiPostMapping(value="batchSyncGoods")
    public CustomApiResult<@ApiResponseBody("Body")Map<String,Object>> costCenterApiHandler(@Valid @ApiParam("入口参数") CostCenterApiReqModel reqModel) {
        String reqData=reqModel.getData();
        JSONArray reqList = JSONObject.parseArray(reqData);

        //获取系统现有商品数据,并且将系统中的启用禁用状态、编码分别放入两个map,分组判断时从map中获取数据
        Map<String,String> sysEnableStatusMap = new HashMap<String,String>();
        Map<String,String> entityCodeMap = new HashMap<String,String>();
        Map<String,String> mainIdMap = new HashMap<String,String>();
        DynamicObjectCollection systemGoodsList = QueryServiceHelper.query("bd_material", "number,enable", new QFilter[]{});
        for(DynamicObject sysGoodsData : systemGoodsList){
            sysEnableStatusMap.put(sysGoodsData.getString("number"),sysGoodsData.getString("enable"));
        }


        JSONArray saveList = new JSONArray();       //只需要进行save操作的数据
        JSONArray enableList = new JSONArray();     //启用数据,先进行启用操作，再save
        JSONArray disableList = new JSONArray();    //禁用数据，直接禁用操作，不需要save
        JSONArray doneList = new JSONArray();       //不需要进行任何操作的数据，返回时直接认为操作成功
        List<String> disableIds = new ArrayList<String>();
        List<String> enableIds = new ArrayList<String>();

        //数据分组，根据系统中是否已存在、启用、禁用状态进行分组
        for(int i=0; i<reqList.size(); i++){
            JSONObject reqJson = reqList.getJSONObject(i);
            String number = reqJson.getString("number");
            entityCodeMap.put(number,reqJson.getString("entityCode"));
            mainIdMap.put(number,reqJson.getString("yrpg_main_id"));
            String enableStatus = reqJson.getString("enable");
            //-------------------系统中赞不存在的数据------------------------
            if(sysEnableStatusMap.get(number) == null){
                if("0".equals(enableStatus)){
                    //不存在，且需要禁用的，不做处理，放入doneList
                    doneList.add(reqJson);
                }else{
                    //不存在，且需要启用的，直接save，默认就是开启状态，放入saveList
                    saveList.add(reqJson);
                }
            }else{
                //-------------------系统中已存在的数据---------------------
                if("1".equals(enableStatus) && "1".equals(sysEnableStatusMap.get(number))){
                    //启用，系统中已经是启用状态的,只需要save,放入saveList
                    saveList.add(reqJson);
                } else if ("1".equals(enableStatus) && "0".equals(sysEnableStatusMap.get(number))) {
                    //启用，系统中是禁用状态的,放入enableList
                    enableList.add(reqJson);
                    enableIds.add(number);
                } else if ("0".equals(enableStatus) && "0".equals(sysEnableStatusMap.get(number))) {
                    //禁用，系统中已经是禁用状态的，不需要任何操作，放入doneList
                    doneList.add(reqJson);
                }else if("0".equals(enableStatus) && "1".equals(sysEnableStatusMap.get(number))){
                    //禁用，系统中是启用状态的额，放入disableList
                    disableList.add(reqJson);
                    disableIds.add(number);
                }else{
                    //不满足条件的统一不做任何处理
                    doneList.add(reqJson);
                }
            }
            //-------------------分组判断结束------------------------------
        }

        List<JSONObject> mdMappingsList = new ArrayList<JSONObject>();


        //save数据处理
        if(saveList.size() > 0){
            JSONObject saveJson = new JSONObject();
            saveJson.put("data",saveList);
            OpenApiResult saveReult = OpenApiSdkUtil.invoke("/v2/yrpg/basedata/bd_material/saveGoods", saveJson, new HashMap<>());
            groupList(mdMappingsList,saveReult,entityCodeMap,mainIdMap,"save");
        }

        //禁用数据处理
        if(disableList.size() > 0){
            JSONObject disableJson = new JSONObject();
            JSONObject ids = new JSONObject();
            ids.put("number",disableIds);
            disableJson.put("data",ids);
            OpenApiResult disableReult = OpenApiSdkUtil.invoke("/v2/yrpg/basedata/bd_material/bd_goodsDisable", disableJson, new HashMap<>());
            groupList(mdMappingsList,disableReult,entityCodeMap,mainIdMap,"disable");
        }
        //启用数据处理
        if(enableList.size() > 0) {
            JSONObject enableJson = new JSONObject();
            JSONObject ids = new JSONObject();
            ids.put("number",enableIds);
            enableJson.put("data",ids);
            //调用启用接口
            OpenApiSdkUtil.invoke("/v2/yrpg/basedata/bd_material/bd_goodsEnable", enableJson, new HashMap<>());
            //反审核
            OpenApiSdkUtil.invoke("/v2/yrpg/basedata/bd_material/bd_goodsUnaudit", enableJson, new HashMap<>());
            //调用save接口
            enableJson.put("data",enableList);
            OpenApiResult saveReult = OpenApiSdkUtil.invoke("/v2/yrpg/basedata/bd_material/saveGoods", enableJson, new HashMap<>());
            groupList(mdMappingsList, saveReult, entityCodeMap,mainIdMap, "save");
        }

        //不需要操作的数据直接封装成功返回
        if(doneList.size()>0){
            for(int i=0;i<doneList.size();i++){
                JSONObject reqJson = doneList.getJSONObject(i);
                JSONObject json = new JSONObject();
                json.put("mdmCode",reqJson.getString("yrpg_main_id"));
                json.put("entityCode",reqJson.getString("entityCode"));
                json.put("busiDataId","");
                json.put("message","该条主数据消费成功！");
                json.put("success",true);
                mdMappingsList.add(json);
            }
        }

        Map<String,Object> result = new HashMap<String,Object>();
        result.put("success",true);
        result.put("message", "消费系统消费这批数据成功！");
        result.put("mdMappings",mdMappingsList);
        return CustomApiResult.success(result);
    }


    private void groupList(List<JSONObject> mdMappingsList,OpenApiResult result,Map<String,String>entityCodeMap,Map<String,String>mainIdMap,String operateType){
        if("save".equals(operateType)){
            ApiSaveResult data = (ApiSaveResult)result.getData();
            List<ApiSaveItemData> saveItemList = data.getResult();
            for(ApiSaveItemData item : saveItemList){
                JSONObject json = new JSONObject();
                String mdmCode = item.getKeys().get("number");
                json.put("mdmCode",mainIdMap.get(mdmCode));
                json.put("entityCode",entityCodeMap.get(mdmCode));
                json.put("busiDataId",item.getId());
                json.put("success",item.isBillStatus());
                if(item.isBillStatus()){
                    json.put("message","该条主数据消费成功！");
                }else{
                    StringBuffer errmsg = new StringBuffer();
                    for(ApiSaveRowErrorData errer:item.getErrors()){
                        //明细错误拼接
                        Set<String> rowMsg = errer.getRowMsg();
                        for(String err:rowMsg){
                            errmsg.append(err);
                        }
                    }
                    json.put("message",errmsg.toString());

                }
                mdMappingsList.add(json);
            }
        }else{
            BaseFilterResult data = (BaseFilterResult)result.getData();
            List<BaseFilterItemData> itemList = data.getResult();
            for(BaseFilterItemData item : itemList){
                JSONObject json = new JSONObject();
                String mdmCode = item.getNumber();
                json.put("mdmCode",mainIdMap.get(mdmCode));
                json.put("entityCode",entityCodeMap.get(mdmCode));
                json.put("busiDataId",item.getId());
                json.put("success",item.isBillStatus());
                if(item.isBillStatus()){
                    json.put("message","该条主数据消费成功！");
                }else{
                    StringBuffer errmsg = new StringBuffer();
                    List<String> errors = item.getErrors();
                    for(String err : errors){
                        errmsg.append(err);
                    }
                    json.put("message",errmsg.toString());
                }
                mdMappingsList.add(json);
            }

        }
    }
}
